@@ -9,10 +9,12 @@ import functools
 import itertools
 import logging
 import pathlib
+import random
 import threading
 import uuid
 from contextvars import copy_context
 from typing import (
+    Awaitable,
     Callable,
     DefaultDict,
     Dict,
@@ -28,7 +30,6 @@ from typing import (
     cast,
 )
 
-from requests import HTTPError
 from typing_extensions import TypedDict
 
 import langsmith
@@ -37,9 +38,11 @@ from langsmith import run_helpers as rh
 from langsmith import run_trees, schemas
 from langsmith import utils as ls_utils
 from langsmith.evaluation.evaluator import (
+    ComparisonEvaluationResult,
     EvaluationResult,
     EvaluationResults,
     RunEvaluator,
+    comparison_evaluator,
     run_evaluator,
 )
 from langsmith.evaluation.integrations import LangChainStringEvaluator
@@ -51,14 +54,29 @@ TARGET_T = Callable[[dict], dict]
 DATA_T = Union[str, uuid.UUID, Iterable[schemas.Example]]
 # Summary evaluator runs over the whole dataset
 # and reports aggregate metric(s)
-SUMMARY_EVALUATOR_T = Callable[
-    [Sequence[schemas.Run], Sequence[schemas.Example]],
-    Union[EvaluationResult, EvaluationResults],
+SUMMARY_EVALUATOR_T = Union[
+    Callable[
+        [Sequence[schemas.Run], Sequence[schemas.Example]],
+        Union[EvaluationResult, EvaluationResults],
+    ],
+    Callable[
+        [List[schemas.Run], List[schemas.Example]],
+        Union[EvaluationResult, EvaluationResults],
+    ],
 ]
 # Row-level evaluator
 EVALUATOR_T = Union[
     RunEvaluator,
-    Callable[[schemas.Run, Optional[schemas.Example]], EvaluationResult],
+    Callable[
+        [schemas.Run, Optional[schemas.Example]],
+        Union[EvaluationResult, EvaluationResults],
+    ],
+]
+AEVALUATOR_T = Union[
+    Callable[
+        [schemas.Run, Optional[schemas.Example]],
+        Awaitable[Union[EvaluationResult, EvaluationResults]],
+    ],
 ]
 
 
@@ -70,7 +88,9 @@ def evaluate(
     summary_evaluators: Optional[Sequence[SUMMARY_EVALUATOR_T]] = None,
     metadata: Optional[dict] = None,
     experiment_prefix: Optional[str] = None,
+    description: Optional[str] = None,
     max_concurrency: Optional[int] = None,
+    num_repetitions: int = 1,
     client: Optional[langsmith.Client] = None,
     blocking: bool = True,
 ) -> ExperimentResults:
@@ -88,12 +108,16 @@ def evaluate(
             Defaults to None.
         experiment_prefix (Optional[str]): A prefix to provide for your experiment name.
             Defaults to None.
+        description (Optional[str]): A free-form text description for the experiment.
         max_concurrency (Optional[int]): The maximum number of concurrent
             evaluations to run. Defaults to None.
         client (Optional[langsmith.Client]): The LangSmith client to use.
             Defaults to None.
         blocking (bool): Whether to block until the evaluation is complete.
             Defaults to True.
+        num_repetitions (int): The number of times to run the evaluation.
+            Each item in the dataset will be run and evaluated this many times.
+            Defaults to 1.
 
     Returns:
         ExperimentResults: The results of the evaluation.
@@ -135,6 +159,8 @@ def evaluate(
         ...     data=dataset_name,
         ...     evaluators=[accuracy],
         ...     summary_evaluators=[precision],
+        ...     experiment_prefix="My Experiment",
+        ...     description="Evaluating the accuracy of a simple prediction model.",
         ...     metadata={
         ...         "my-prompt-version": "abcd-1234",
         ...     },
@@ -151,6 +177,7 @@ def evaluate(
         ...     evaluators=[accuracy],
         ...     summary_evaluators=[precision],
         ...     experiment_prefix="My Experiment",
+        ...     description="Just testing a subset synchronously.",
         ... )  # doctest: +ELLIPSIS
         View the evaluation results for experiment:...
 
@@ -161,6 +188,7 @@ def evaluate(
         ...     data=dataset_name,
         ...     evaluators=[accuracy],
         ...     summary_evaluators=[precision],
+        ...     description="I don't even have to block!",
         ...     blocking=False,
         ... )  # doctest: +ELLIPSIS
         View the evaluation results for experiment:...
@@ -170,6 +198,7 @@ def evaluate(
         Using the `evaluate` API with an off-the-shelf LangChain evaluator:
 
         >>> from langsmith.evaluation import LangChainStringEvaluator
+        >>> from langchain_openai import ChatOpenAI
         >>> def prepare_criteria_data(run: Run, example: Example):
         ...     return {
         ...         "prediction": run.outputs["output"],
@@ -189,10 +218,12 @@ def evaluate(
         ...                     "usefulness": "The prediction is useful if it is correct"
         ...                     " and/or asks a useful followup question."
         ...                 },
+        ...                 "llm": ChatOpenAI(model="gpt-4o"),
         ...             },
         ...             prepare_data=prepare_criteria_data,
         ...         ),
         ...     ],
+        ...     description="Evaluating with off-the-shelf LangChain evaluators.",
         ...     summary_evaluators=[precision],
         ... )  # doctest: +ELLIPSIS
         View the evaluation results for experiment:...
@@ -210,6 +241,7 @@ def evaluate(
         ...     lc_predict.invoke,
         ...     data=dataset_name,
         ...     evaluators=[accuracy],
+        ...     description="This time we're evaluating a LangChain object.",
         ...     summary_evaluators=[precision],
         ... )  # doctest: +ELLIPSIS
         View the evaluation results for experiment:...
@@ -221,7 +253,9 @@ def evaluate(
         summary_evaluators=summary_evaluators,
         metadata=metadata,
         experiment_prefix=experiment_prefix,
+        description=description,
         max_concurrency=max_concurrency,
+        num_repetitions=num_repetitions,
         client=client,
         blocking=blocking,
     )
@@ -302,14 +336,8 @@ def evaluate_existing(
     client = client or langsmith.Client()
     project = _load_experiment(experiment, client)
     runs = _load_traces(experiment, client, load_nested=load_nested)
-    data = list(
-        client.list_examples(
-            dataset_id=project.reference_dataset_id,
-            as_of=project.metadata.get("dataset_version"),
-        )
-    )
-    runs = sorted(runs, key=lambda r: str(r.reference_example_id))
-    data = sorted(data, key=lambda d: str(d.id))
+    data_map = _load_examples_map(client, project)
+    data = [data_map[cast(uuid.UUID, run.reference_example_id)] for run in runs]
     return _evaluate(
         runs,
         data=data,
@@ -319,6 +347,7 @@ def evaluate_existing(
         max_concurrency=max_concurrency,
         client=client,
         blocking=blocking,
+        experiment=project,
     )
 
 
@@ -391,9 +420,352 @@ class ExperimentResults:
         self._thread.join()
 
 
+## Public API for Comparison Experiments
+
+# Row-level evaluator
+COMPARATIVE_EVALUATOR_T = Callable[
+    [Sequence[schemas.Run], Optional[schemas.Example]],
+    Union[
+        Union[ComparisonEvaluationResult, dict],
+        Awaitable[Union[ComparisonEvaluationResult, dict]],
+    ],
+]
+
+
+def evaluate_comparative(
+    experiments: Tuple[Union[str, uuid.UUID], Union[str, uuid.UUID]],
+    /,
+    evaluators: Sequence[COMPARATIVE_EVALUATOR_T],
+    experiment_prefix: Optional[str] = None,
+    description: Optional[str] = None,
+    max_concurrency: int = 5,
+    client: Optional[langsmith.Client] = None,
+    metadata: Optional[dict] = None,
+    load_nested: bool = False,
+    randomize_order: bool = False,
+) -> ComparativeExperimentResults:
+    r"""Evaluate existing experiment runs against each other.
+
+    This lets you use pairwise preference scoring to generate more
+    reliable feedback in your experiments.
+
+    Args:
+        experiments (Tuple[Union[str, uuid.UUID], Union[str, uuid.UUID]]):
+            The identifiers of the experiments to compare.
+        evaluators (Sequence[COMPARATIVE_EVALUATOR_T]):
+            A list of evaluators to run on each example.
+        experiment_prefix (Optional[str]): A prefix to provide for your experiment name.
+            Defaults to None.
+        description (Optional[str]): A free-form text description for the experiment.
+        max_concurrency (int): The maximum number of concurrent evaluations to run.
+            Defaults to 5.
+        client (Optional[langsmith.Client]): The LangSmith client to use.
+            Defaults to None.
+        metadata (Optional[dict]): Metadata to attach to the experiment.
+            Defaults to None.
+        load_nested (bool): Whether to load all child runs for the experiment.
+            Default is to only load the top-level root runs.
+        randomize_order (bool): Whether to randomize the order of the outputs for each evaluation.
+            Default is False.
+
+    Returns:
+        ComparativeExperimentResults: The results of the comparative evaluation.
+
+    Examples:
+        Suppose you want to compare two prompts to see which one is more effective.
+        You would first prepare your dataset:
+
+        >>> from typing import Sequence
+        >>> from langsmith import Client
+        >>> from langsmith.evaluation import evaluate
+        >>> from langsmith.schemas import Example, Run
+        >>> client = Client()
+        >>> client.clone_public_dataset(
+        ...     "https://smith.langchain.com/public/419dcab2-1d66-4b94-8901-0357ead390df/d"
+        ... )
+        >>> dataset_name = "Evaluate Examples"
+
+        Then you would run your different prompts:
+        >>> import functools
+        >>> import openai
+        >>> from langsmith.evaluation import evaluate
+        >>> from langsmith.wrappers import wrap_openai
+        >>> oai_client = openai.Client()
+        >>> wrapped_client = wrap_openai(oai_client)
+        >>> prompt_1 = "You are a helpful assistant."
+        >>> prompt_2 = "You are an exceedingly helpful assistant."
+        >>> def predict(inputs: dict, prompt: str) -> dict:
+        ...     completion = wrapped_client.chat.completions.create(
+        ...         model="gpt-3.5-turbo",
+        ...         messages=[
+        ...             {"role": "system", "content": prompt},
+        ...             {
+        ...                 "role": "user",
+        ...                 "content": f"Context: {inputs['context']}"
+        ...                 f"\n\ninputs['question']",
+        ...             },
+        ...         ],
+        ...     )
+        ...     return {"output": completion.choices[0].message.content}
+        >>> results_1 = evaluate(
+        ...     functools.partial(predict, prompt=prompt_1),
+        ...     data=dataset_name,
+        ...     description="Evaluating our basic system prompt.",
+        ...     blocking=False,  # Run these experiments in parallel
+        ... )  # doctest: +ELLIPSIS
+        View the evaluation results for experiment:...
+        >>> results_2 = evaluate(
+        ...     functools.partial(predict, prompt=prompt_2),
+        ...     data=dataset_name,
+        ...     description="Evaluating our advanced system prompt.",
+        ...     blocking=False,
+        ... )  # doctest: +ELLIPSIS
+        View the evaluation results for experiment:...
+        >>> results_1.wait()
+        >>> results_2.wait()
+        >>> import time
+        >>> time.sleep(10)  # Wait for the traces to be fully processed
+
+            Finally, you would compare the two prompts directly:
+        >>> import json
+        >>> from langsmith.evaluation import evaluate_comparative
+        >>> def score_preferences(runs: list, example):
+        ...     pred_a = runs[0].outputs["output"]
+        ...     pred_b = runs[1].outputs["output"]
+        ...     ground_truth = example.outputs["answer"]
+        ...     tools = [
+        ...         {
+        ...             "type": "function",
+        ...             "function": {
+        ...                 "name": "rank_preferences",
+        ...                 "description": "Saves the prefered response ('A' or 'B')",
+        ...                 "parameters": {
+        ...                     "type": "object",
+        ...                     "properties": {
+        ...                         "reasoning": {
+        ...                             "type": "string",
+        ...                             "description": "The reasoning behind the choice.",
+        ...                         },
+        ...                         "preferred_option": {
+        ...                             "type": "string",
+        ...                             "enum": ["A", "B"],
+        ...                             "description": "The preferred option, either 'A' or 'B'",
+        ...                         },
+        ...                     },
+        ...                     "required": ["preferred_option"],
+        ...                 },
+        ...             },
+        ...         }
+        ...     ]
+        ...     completion = openai.Client().chat.completions.create(
+        ...         model="gpt-3.5-turbo",
+        ...         messages=[
+        ...             {"role": "system", "content": "Select the better response."},
+        ...             {
+        ...                 "role": "user",
+        ...                 "content": f"Option A: {pred_a}"
+        ...                 f"\n\nOption B: {pred_b}"
+        ...                 f"\n\nGround Truth: {ground_truth}",
+        ...             },
+        ...         ],
+        ...         tools=tools,
+        ...         tool_choice={
+        ...             "type": "function",
+        ...             "function": {"name": "rank_preferences"},
+        ...         },
+        ...     )
+        ...     tool_args = completion.choices[0].message.tool_calls[0].function.arguments
+        ...     preference = json.loads(tool_args)["preferred_option"]
+        ...     if preference == "A":
+        ...         return {
+        ...             "key": "ranked_preference",
+        ...             "scores": {runs[0].id: 1, runs[1].id: 0},
+        ...         }
+        ...     else:
+        ...         return {
+        ...             "key": "ranked_preference",
+        ...             "scores": {runs[0].id: 0, runs[1].id: 1},
+        ...         }
+        >>> results = evaluate_comparative(
+        ...     [results_1.experiment_name, results_2.experiment_name],
+        ...     evaluators=[score_preferences],
+        ...     client=client,
+        ... )  # doctest: +ELLIPSIS
+        View the pairwise evaluation results at:...
+    """  # noqa: E501
+    if len(experiments) < 2:
+        raise ValueError("Comparative evaluation requires at least 2 experiments.")
+    if not evaluators:
+        raise ValueError(
+            "At least one evaluator is required for comparative evaluation."
+        )
+    if max_concurrency < 0:
+        raise ValueError("max_concurrency must be a positive integer.")
+    client = client or langsmith.Client()
+
+    # TODO: Add information about comparison experiments
+    projects = [_load_experiment(experiment, client) for experiment in experiments]
+    ref_datasets_ = [str(p.reference_dataset_id) for p in projects]
+    if not len(set(ref_datasets_)) == 1:
+        raise ValueError("All experiments must have the same reference dataset.")
+    experiment_ids = [p.id for p in projects]
+    if experiment_prefix is None:
+        experiment_names = [p.name for p in projects if p.name is not None]
+        experiment_name = (
+            " vs. ".join(experiment_names) + "-" + str(uuid.uuid4().hex[:4])
+        )
+    else:
+        experiment_name = experiment_prefix + "-" + str(uuid.uuid4().hex[:8])
+    comparative_experiment_id = uuid.uuid4()
+    comparative_experiment = client.create_comparative_experiment(
+        experiment_name,
+        experiments=experiment_ids,
+        description=description,
+        metadata=metadata,
+        id=comparative_experiment_id,
+    )
+    _print_comparative_experiment_start(
+        cast(
+            Tuple[schemas.TracerSessionResult, schemas.TracerSessionResult],
+            tuple(projects),
+        ),
+        comparative_experiment,
+    )
+    runs = [
+        _load_traces(experiment, client, load_nested=load_nested)
+        for experiment in experiments
+    ]
+    # Only check intersections for the experiments
+    examples_intersection = None
+    for runs_list in runs:
+        example_ids_set = {run.reference_example_id for run in runs_list}
+        if examples_intersection is None:
+            examples_intersection = example_ids_set
+        else:
+            examples_intersection &= example_ids_set
+    example_ids_nullable = (
+        list(examples_intersection) if examples_intersection is not None else []
+    )
+    example_ids = [eid for eid in example_ids_nullable if eid is not None]
+    # TODO: Warn if different dataset versions, etc. are used in the different
+    # experiments. We aren't providing any training wheels here.
+    batch_size = 99
+    data = {}
+    for i in range(0, len(example_ids), batch_size):
+        example_ids_batch = example_ids[i : i + batch_size]
+        for e in client.list_examples(
+            dataset_id=projects[0].reference_dataset_id,
+            as_of=projects[0].metadata.get("dataset_version"),
+            example_ids=example_ids_batch,
+        ):
+            data[e.id] = e
+    runs_dict: Dict[uuid.UUID, List[schemas.Run]] = collections.defaultdict(list)
+    for runs_list in runs:
+        for run in runs_list:
+            if run.reference_example_id in data:
+                runs_dict[cast(uuid.UUID, run.reference_example_id)].append(run)
+
+    comparators = [comparison_evaluator(evaluator) for evaluator in evaluators or []]
+    results: dict = {}
+
+    def evaluate_and_submit_feedback(
+        runs_list: list[schemas.Run], example: schemas.Example, executor: cf.Executor
+    ) -> ComparisonEvaluationResult:
+        feedback_group_id = uuid.uuid4()
+        if randomize_order:
+            random.shuffle(runs_list)
+        result = comparator.compare_runs(runs_list, example)
+        if client is None:
+            raise ValueError("Client is required to submit feedback.")
+        for run_id, score in result.scores.items():
+            executor.submit(
+                client.create_feedback,
+                run_id=run_id,
+                key=result.key,
+                score=score,
+                comparative_experiment_id=comparative_experiment.id,
+                source_run_id=result.source_run_id,
+                feedback_group_id=feedback_group_id,
+            )
+        return result
+
+    tqdm = _load_tqdm()
+    with ls_utils.ContextThreadPoolExecutor(
+        max_workers=max_concurrency or 1
+    ) as executor:
+        futures = []
+        for example_id, runs_list in tqdm(runs_dict.items()):
+            results[example_id] = {
+                "runs": runs_list,
+            }
+            for comparator in comparators:
+                if max_concurrency > 1:
+                    future = executor.submit(
+                        evaluate_and_submit_feedback,
+                        runs_list,
+                        data[example_id],
+                        executor,
+                    )
+                    futures.append(future)
+                else:
+                    result = evaluate_and_submit_feedback(
+                        runs_list, data[example_id], executor
+                    )
+                    results[example_id][f"feedback.{result.key}"] = result
+            if futures:
+                cf.wait(futures)
+                for future in futures:
+                    result = future.result()
+                    results[example_id][f"feedback.{result.key}"] = result
+
+    return ComparativeExperimentResults(results)
+
+
+class ComparativeExperimentResults:
+    """Represents the results of an evaluate_comparative() call.
+
+    This class provides an iterator interface to iterate over the experiment results
+    as they become available. It also provides methods to access the experiment name,
+    the number of results, and to wait for the results to be processed.
+
+    Methods:
+        experiment_name() -> str: Returns the name of the experiment.
+        wait() -> None: Waits for the experiment data to be processed.
+    """
+
+    def __init__(
+        self,
+        results: dict,
+    ):
+        self._results = results
+
+    def __getitem__(self, key):
+        """Return the result associated with the given key."""
+        return self._results[key]
+
+
 ## Private API
 
 
+def _print_comparative_experiment_start(
+    experiments: Tuple[schemas.TracerSession, schemas.TracerSession],
+    comparative_experiment: schemas.ComparativeExperiment,
+) -> None:
+    url = experiments[0].url or experiments[1].url
+    if url:
+        project_url = url.split("?")[0]
+        dataset_id = comparative_experiment.reference_dataset_id
+        base_url = project_url.split("/projects/p/")[0]
+        comparison_url = (
+            f"{base_url}/datasets/{dataset_id}/compare?"
+            f"selectedSessions={'%2C'.join([str(e.id) for e in experiments])}"
+            f"&comparativeExperiment={comparative_experiment.id}"
+        )
+        print(  # noqa: T201
+            f"View the pairwise evaluation results at:\n{comparison_url}\n\n"
+        )
+
+
 def _is_callable(target: Union[TARGET_T, Iterable[schemas.Run]]) -> bool:
     return callable(target) or (hasattr(target, "invoke") and callable(target.invoke))
 
@@ -406,7 +778,9 @@ def _evaluate(
     summary_evaluators: Optional[Sequence[SUMMARY_EVALUATOR_T]] = None,
     metadata: Optional[dict] = None,
     experiment_prefix: Optional[str] = None,
+    description: Optional[str] = None,
     max_concurrency: Optional[int] = None,
+    num_repetitions: int = 1,
     client: Optional[langsmith.Client] = None,
     blocking: bool = True,
     experiment: Optional[schemas.TracerSession] = None,
@@ -425,6 +799,8 @@ def _evaluate(
         client=client,
         metadata=metadata,
         experiment=experiment_ or experiment_prefix,
+        description=description,
+        num_repetitions=num_repetitions,
         # If provided, we don't need to create a new experiment.
         runs=runs,
         # Create or resolve the experiment.
@@ -497,6 +873,18 @@ def _load_traces(
     return results
 
 
+def _load_examples_map(
+    client: langsmith.Client, project: schemas.TracerSession
+) -> Dict[uuid.UUID, schemas.Example]:
+    return {
+        e.id: e
+        for e in client.list_examples(
+            dataset_id=project.reference_dataset_id,
+            as_of=project.metadata.get("dataset_version"),
+        )
+    }
+
+
 IT = TypeVar("IT")
 
 
@@ -518,6 +906,7 @@ class _ExperimentManagerMixin:
         experiment: Optional[Union[schemas.TracerSession, str]],
         metadata: Optional[dict] = None,
         client: Optional[langsmith.Client] = None,
+        description: Optional[str] = None,
     ):
         self.client = client or langsmith.Client()
         self._experiment: Optional[schemas.TracerSession] = None
@@ -538,6 +927,7 @@ class _ExperimentManagerMixin:
                 **metadata,
             }
         self._metadata = metadata or {}
+        self._description = description
 
     @property
     def experiment_name(self) -> str:
@@ -567,23 +957,33 @@ class _ExperimentManagerMixin:
             }
         return project_metadata
 
+    def _create_experiment(
+        self, dataset_id: uuid.UUID, metadata: dict
+    ) -> schemas.TracerSession:
+        # There is a chance of name collision, so we'll retry
+        starting_name = self._experiment_name
+        num_attempts = 10
+        for _ in range(num_attempts):
+            try:
+                return self.client.create_project(
+                    self._experiment_name,
+                    description=self._description,
+                    reference_dataset_id=dataset_id,
+                    metadata=metadata,
+                )
+            except ls_utils.LangSmithConflictError:
+                self._experiment_name = f"{starting_name}-{str(uuid.uuid4().hex[:6])}"
+        raise ValueError(
+            f"Could not find a unique experiment name in {num_attempts} attempts."
+            " Please try again with a different experiment name."
+        )
+
     def _get_project(self, first_example: schemas.Example) -> schemas.TracerSession:
         if self._experiment is None:
-            try:
-                project_metadata = self._get_experiment_metadata()
-                project = self.client.create_project(
-                    self.experiment_name,
-                    reference_dataset_id=first_example.dataset_id,
-                    metadata=project_metadata,
-                )
-            except (HTTPError, ValueError, ls_utils.LangSmithError) as e:
-                if "already exists " not in str(e):
-                    raise e
-                raise ValueError(
-                    # TODO: Better error
-                    f"Experiment {self.experiment_name} already exists."
-                    " Please use a different name."
-                )
+            project_metadata = self._get_experiment_metadata()
+            project = self._create_experiment(
+                first_example.dataset_id, project_metadata
+            )
         else:
             project = self._experiment
         return project
@@ -618,6 +1018,7 @@ class _ExperimentManager(_ExperimentManagerMixin):
     Args:
         data (DATA_T): The data used for the experiment. Can be a dataset name or ID OR
             a generator of examples.
+        num_repetitions (int): The number of times to run over the data.
         runs (Optional[Iterable[schemas.Run]]): The runs associated with the experiment
             predictions.
         experiment (Optional[schemas.TracerSession]): The tracer session
@@ -642,22 +1043,30 @@ class _ExperimentManager(_ExperimentManagerMixin):
         runs: Optional[Iterable[schemas.Run]] = None,
         evaluation_results: Optional[Iterable[EvaluationResults]] = None,
         summary_results: Optional[Iterable[EvaluationResults]] = None,
+        description: Optional[str] = None,
+        num_repetitions: int = 1,
     ):
         super().__init__(
             experiment=experiment,
             metadata=metadata,
             client=client,
+            description=description,
         )
         self._data = data
         self._examples: Optional[Iterable[schemas.Example]] = None
         self._runs = runs
         self._evaluation_results = evaluation_results
         self._summary_results = summary_results
+        self._num_repetitions = num_repetitions
 
     @property
     def examples(self) -> Iterable[schemas.Example]:
         if self._examples is None:
             self._examples = _resolve_data(self._data, client=self.client)
+            if self._num_repetitions > 1:
+                self._examples = itertools.chain.from_iterable(
+                    itertools.tee(self._examples, self._num_repetitions)
+                )
         self._examples, examples_iter = itertools.tee(self._examples)
         return examples_iter
 
@@ -691,6 +1100,7 @@ class _ExperimentManager(_ExperimentManagerMixin):
         first_example = next(itertools.islice(self.examples, 1))
         project = self._get_project(first_example)
         self._print_experiment_start(project, first_example)
+        self._metadata["num_repetitions"] = self._num_repetitions
         return self.__class__(
             self.examples,
             experiment=project,
@@ -809,7 +1219,7 @@ class _ExperimentManager(_ExperimentManagerMixin):
                 )
 
         else:
-            with cf.ThreadPoolExecutor(max_concurrency) as executor:
+            with ls_utils.ContextThreadPoolExecutor(max_concurrency) as executor:
                 futures = [
                     executor.submit(
                         _forward,
@@ -841,7 +1251,12 @@ class _ExperimentManager(_ExperimentManagerMixin):
             },
         }
         with rh.tracing_context(
-            **{**current_context, "project_name": "evaluators", "metadata": metadata}
+            **{
+                **current_context,
+                "project_name": "evaluators",
+                "metadata": metadata,
+                "enabled": True,
+            }
         ):
             run = current_results["run"]
             example = current_results["example"]
@@ -882,10 +1297,13 @@ class _ExperimentManager(_ExperimentManagerMixin):
         (e.g. from a previous prediction step)
         """
         if max_concurrency == 0:
+            context = copy_context()
             for current_results in self.get_results():
-                yield self._run_evaluators(evaluators, current_results)
+                yield context.run(self._run_evaluators, evaluators, current_results)
         else:
-            with cf.ThreadPoolExecutor(max_workers=max_concurrency) as executor:
+            with ls_utils.ContextThreadPoolExecutor(
+                max_workers=max_concurrency
+            ) as executor:
                 futures = []
                 for current_results in self.get_results():
                     futures.append(
@@ -907,7 +1325,7 @@ class _ExperimentManager(_ExperimentManagerMixin):
             runs.append(run)
             examples.append(example)
         aggregate_feedback = []
-        with cf.ThreadPoolExecutor() as executor:
+        with ls_utils.ContextThreadPoolExecutor() as executor:
             project_id = self._get_experiment().id
             current_context = rh.get_tracing_context()
             metadata = {
@@ -957,6 +1375,23 @@ class _ExperimentManager(_ExperimentManagerMixin):
         max_modified_at = max(modified_at) if modified_at else None
         return max_modified_at.isoformat() if max_modified_at else None
 
+    def _get_dataset_splits(self) -> Optional[list[str]]:
+        examples = list(self.examples)
+        splits = set()
+        for example in examples:
+            if (
+                example.metadata
+                and example.metadata.get("dataset_split")
+                and isinstance(example.metadata["dataset_split"], list)
+            ):
+                for split in example.metadata["dataset_split"]:
+                    if isinstance(split, str):
+                        splits.add(split)
+            else:
+                splits.add("base")
+
+        return list(splits)
+
     def _end(self) -> None:
         experiment = self._experiment
         if experiment is None:
@@ -964,6 +1399,7 @@ class _ExperimentManager(_ExperimentManagerMixin):
 
         project_metadata = self._get_experiment_metadata()
         project_metadata["dataset_version"] = self._get_dataset_version()
+        project_metadata["dataset_splits"] = self._get_dataset_splits()
         self.client.update_project(
             experiment.id,
             end_time=datetime.datetime.now(datetime.timezone.utc),
@@ -972,7 +1408,7 @@ class _ExperimentManager(_ExperimentManagerMixin):
 
 
 def _resolve_evaluators(
-    evaluators: Sequence[EVALUATOR_T],
+    evaluators: Sequence[Union[EVALUATOR_T, RunEvaluator, AEVALUATOR_T]],
 ) -> Sequence[RunEvaluator]:
     results = []
     for evaluator in evaluators:
@@ -999,7 +1435,7 @@ def _wrap_summary_evaluators(
             def _wrapper_super_inner(
                 runs_: str, examples_: str
             ) -> Union[EvaluationResult, EvaluationResults]:
-                return evaluator(runs, examples)
+                return evaluator(list(runs), list(examples))
 
             return _wrapper_super_inner(
                 f"Runs[] (Length={len(runs)})", f"Examples[] (Length={len(examples)})"
@@ -1031,30 +1467,31 @@ def _forward(
         nonlocal run
         run = r
 
-    try:
-        fn(
-            example.inputs,
-            langsmith_extra=rh.LangSmithExtra(
-                reference_example_id=example.id,
-                on_end=_get_run,
-                project_name=experiment_name,
-                metadata={
-                    **metadata,
-                    "example_version": (
-                        example.modified_at.isoformat()
-                        if example.modified_at
-                        else example.created_at.isoformat()
-                    ),
-                },
-                client=client,
-            ),
+    with rh.tracing_context(enabled=True):
+        try:
+            fn(
+                example.inputs,
+                langsmith_extra=rh.LangSmithExtra(
+                    reference_example_id=example.id,
+                    on_end=_get_run,
+                    project_name=experiment_name,
+                    metadata={
+                        **metadata,
+                        "example_version": (
+                            example.modified_at.isoformat()
+                            if example.modified_at
+                            else example.created_at.isoformat()
+                        ),
+                    },
+                    client=client,
+                ),
+            )
+        except Exception as e:
+            logger.error(f"Error running target function: {e}")
+        return _ForwardResults(
+            run=cast(schemas.Run, run),
+            example=example,
         )
-    except Exception as e:
-        logger.error(f"Error running target function: {e}")
-    return _ForwardResults(
-        run=cast(schemas.Run, run),
-        example=example,
-    )
 
 
 def _resolve_data(
@@ -1068,12 +1505,14 @@ def _resolve_data(
     return data
 
 
-def _ensure_traceable(target: TARGET_T) -> rh.SupportsLangsmithExtra:
+def _ensure_traceable(
+    target: TARGET_T | rh.SupportsLangsmithExtra[[dict], dict],
+) -> rh.SupportsLangsmithExtra[[dict], dict]:
     """Ensure the target function is traceable."""
     if not callable(target):
         raise ValueError("Target must be a callable function.")
     if rh.is_traceable_function(target):
-        fn = cast(rh.SupportsLangsmithExtra, target)
+        fn = target
     else:
         fn = rh.traceable(name="Target")(target)
     return fn
@@ -1090,7 +1529,7 @@ def _resolve_experiment(
     if experiment is not None:
         if not experiment.name:
             raise ValueError("Experiment name must be defined if provided.")
-        return experiment, None
+        return experiment, runs
     # If we have runs, that means the experiment was already started.
     if runs is not None:
         if runs is not None:
