@@ -1,6 +1,9 @@
 """Generic utility functions."""
 
+from __future__ import annotations
+
 import contextlib
+import contextvars
 import copy
 import enum
 import functools
@@ -11,11 +14,14 @@ import subprocess
 import sys
 import threading
 import traceback
+from concurrent.futures import Future, ThreadPoolExecutor
 from typing import (
     Any,
     Callable,
     Dict,
     Generator,
+    Iterable,
+    Iterator,
     List,
     Mapping,
     Optional,
@@ -23,9 +29,12 @@ from typing import (
     Tuple,
     TypeVar,
     Union,
+    cast,
 )
 
+import httpx
 import requests
+from typing_extensions import ParamSpec
 from urllib3.util import Retry
 
 from langsmith import schemas as ls_schemas
@@ -65,11 +74,11 @@ class LangSmithConnectionError(LangSmithError):
     """Couldn't connect to the LangSmith API."""
 
 
-def tracing_is_enabled() -> bool:
+def tracing_is_enabled(ctx: Optional[dict] = None) -> bool:
     """Return True if tracing is enabled."""
     from langsmith.run_helpers import get_current_run_tree, get_tracing_context
 
-    tc = get_tracing_context()
+    tc = ctx or get_tracing_context()
     # You can manually override the environment using context vars.
     # Check that first.
     # Doing this before checking the run tree lets us
@@ -115,13 +124,18 @@ def xor_args(*arg_groups: Tuple[str, ...]) -> Callable:
     return decorator
 
 
-def raise_for_status_with_text(response: requests.Response) -> None:
+def raise_for_status_with_text(
+    response: Union[requests.Response, httpx.Response],
+) -> None:
     """Raise an error with the response text."""
     try:
         response.raise_for_status()
     except requests.HTTPError as e:
         raise requests.HTTPError(str(e), response.text) from e  # type: ignore[call-arg]
 
+    except httpx.HTTPError as e:
+        raise httpx.HTTPError(str(e), response.text) from e  # type: ignore[call-arg]
+
 
 def get_enum_value(enu: Union[enum.Enum, str]) -> str:
     """Get the value of a string enum."""
@@ -561,3 +575,143 @@ def deepish_copy(val: T) -> T:
         # what we can
         _LOGGER.debug("Failed to deepcopy input: %s", repr(e))
         return _middle_copy(val, memo)
+
+
+def is_version_greater_or_equal(current_version: str, target_version: str) -> bool:
+    """Check if the current version is greater or equal to the target version."""
+    from packaging import version
+
+    current = version.parse(current_version)
+    target = version.parse(target_version)
+    return current >= target
+
+
+def parse_prompt_identifier(identifier: str) -> Tuple[str, str, str]:
+    """Parse a string in the format of owner/name:hash, name:hash, owner/name, or name.
+
+    Args:
+        identifier (str): The prompt identifier to parse.
+
+    Returns:
+        Tuple[str, str, str]: A tuple containing (owner, name, hash).
+
+    Raises:
+        ValueError: If the identifier doesn't match the expected formats.
+    """
+    if (
+        not identifier
+        or identifier.count("/") > 1
+        or identifier.startswith("/")
+        or identifier.endswith("/")
+    ):
+        raise ValueError(f"Invalid identifier format: {identifier}")
+
+    parts = identifier.split(":", 1)
+    owner_name = parts[0]
+    commit = parts[1] if len(parts) > 1 else "latest"
+
+    if "/" in owner_name:
+        owner, name = owner_name.split("/", 1)
+        if not owner or not name:
+            raise ValueError(f"Invalid identifier format: {identifier}")
+        return owner, name, commit
+    else:
+        if not owner_name:
+            raise ValueError(f"Invalid identifier format: {identifier}")
+        return "-", owner_name, commit
+
+
+P = ParamSpec("P")
+
+
+class ContextThreadPoolExecutor(ThreadPoolExecutor):
+    """ThreadPoolExecutor that copies the context to the child thread."""
+
+    def submit(  # type: ignore[override]
+        self,
+        func: Callable[P, T],
+        *args: P.args,
+        **kwargs: P.kwargs,
+    ) -> Future[T]:
+        """Submit a function to the executor.
+
+        Args:
+            func (Callable[..., T]): The function to submit.
+            *args (Any): The positional arguments to the function.
+            **kwargs (Any): The keyword arguments to the function.
+
+        Returns:
+            Future[T]: The future for the function.
+        """
+        return super().submit(
+            cast(
+                Callable[..., T],
+                functools.partial(
+                    contextvars.copy_context().run, func, *args, **kwargs
+                ),
+            )
+        )
+
+    def map(
+        self,
+        fn: Callable[..., T],
+        *iterables: Iterable[Any],
+        timeout: Optional[float] = None,
+        chunksize: int = 1,
+    ) -> Iterator[T]:
+        """Return an iterator equivalent to stdlib map.
+
+        Each function will receive it's own copy of the context from the parent thread.
+
+        Args:
+            fn: A callable that will take as many arguments as there are
+                passed iterables.
+            timeout: The maximum number of seconds to wait. If None, then there
+                is no limit on the wait time.
+            chunksize: The size of the chunks the iterable will be broken into
+                before being passed to a child process. This argument is only
+                used by ProcessPoolExecutor; it is ignored by
+                ThreadPoolExecutor.
+
+        Returns:
+            An iterator equivalent to: map(func, *iterables) but the calls may
+            be evaluated out-of-order.
+
+        Raises:
+            TimeoutError: If the entire result iterator could not be generated
+                before the given timeout.
+            Exception: If fn(*args) raises for any values.
+        """
+        contexts = [contextvars.copy_context() for _ in range(len(iterables[0]))]  # type: ignore[arg-type]
+
+        def _wrapped_fn(*args: Any) -> T:
+            return contexts.pop().run(fn, *args)
+
+        return super().map(
+            _wrapped_fn,
+            *iterables,
+            timeout=timeout,
+            chunksize=chunksize,
+        )
+
+
+def get_api_url(api_url: Optional[str]) -> str:
+    """Get the LangSmith API URL from the environment or the given value."""
+    _api_url = api_url or cast(
+        str,
+        get_env_var(
+            "ENDPOINT",
+            default="https://api.smith.langchain.com",
+        ),
+    )
+    if not _api_url.strip():
+        raise LangSmithUserError("LangSmith API URL cannot be empty")
+    return _api_url.strip().strip('"').strip("'").rstrip("/")
+
+
+def get_api_key(api_key: Optional[str]) -> Optional[str]:
+    """Get the API key from the environment or the given value."""
+    api_key_ = api_key if api_key is not None else get_env_var("API_KEY", default=None)
+    if api_key_ is None or not api_key_.strip():
+        return None
+    return api_key_.strip().strip('"').strip("'")
